// $Id: FileFXMLDocument.java,v 1.1.1.1 2006-03-22 13:32:17 pierre Exp $

package com.senellart.pierre.fuzzyxml.document;

import java.io.File;
import java.io.FileOutputStream;

import javax.xml.parsers.SAXParser;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.senellart.pierre.fuzzyxml.FuzzyXML;
import com.senellart.pierre.fuzzyxml.FuzzyXMLEnvironment;
import com.senellart.pierre.fuzzyxml.exception.FuzzyXMLUnsupportedException;

public class FileFXMLDocument extends FXMLDocument {
	public class GetInfoHandler extends DefaultHandler {
		public void startElement(String uri, String localName, String qName,
				Attributes attributes) throws SAXException {
			String id = attributes.getValue(FuzzyXML.NAMESPACE_URI, "id");
			if (id != null)
				currentId = Math.max(Integer.parseInt(id.substring(1)),
						currentId);

			if (eventsFileName == null)
				eventsFileName = attributes.getValue(FuzzyXML.NAMESPACE_URI,
						"events");
		}
	}

	private DocumentManager documentManager;

	private String fileName;

	private Object mutex;

	private int locked;

	private String eventsFileName;

	private int currentId;

	private Document eventsDoc;

	private int nbEvents;

	public FileFXMLDocument(DocumentManager documentManager, String documentId) {
		this.documentManager = documentManager;
		this.fileName = documentManager.getEnvironment().getPath() + "/"
				+ documentId;
		mutex = new Object();
		locked = 0;

		FuzzyXMLEnvironment environment = documentManager.getEnvironment();

		currentId = -1;
		getInfoFromDocument();

		try {
			eventsDoc = environment.getDOMParser().parse(
					new File(fileName).getParent() + "/" + eventsFileName);
			nbEvents = 0;
			for (Node n = eventsDoc.getDocumentElement().getFirstChild(); n != null; n = n
					.getNextSibling())
				if (n.getNodeType() == Node.ELEMENT_NODE)
					++nbEvents;
		} catch (Exception e) {
			throw new FuzzyXMLUnsupportedException(e);
		}

	}

	private void getInfoFromDocument() {
		try {
			SAXParser saxParser = documentManager.getEnvironment()
					.getSAXFactory().newSAXParser();

			saxParser.parse(new File(fileName), new GetInfoHandler());
		} catch (Exception e) {
			throw new FuzzyXMLUnsupportedException(e);
		}
	}

	public synchronized String getNewId() {
		return "n" + String.valueOf(++currentId);
	}

	public synchronized String addEvent(double confidence, String origin) {
		FuzzyXMLEnvironment environment = documentManager.getEnvironment();

		Element event = eventsDoc.createElementNS(FuzzyXML.NAMESPACE_URI,
				"event");
		event.setAttribute("confidence", String.valueOf(confidence));
		if (origin != null)
			event.setAttribute("origin", origin);

		eventsDoc.getDocumentElement().appendChild(event);

		try {
			environment.serialize(eventsDoc, new FileOutputStream(
					documentManager.getEnvironment().getPath()+"/"+eventsFileName));
		} catch (Exception e) {
			throw new FuzzyXMLUnsupportedException(e);
		}

		return String.valueOf(++nbEvents);
	}

	public String getFileName() {
		return fileName;
	}

	public void getReadLock() throws InterruptedException {
		synchronized (mutex) {
			while (locked == -1) {
				mutex.wait();
			}
			++locked;
		}
	}

	public void getWriteLock() throws InterruptedException {
		synchronized (mutex) {
			while (locked != 0) {
				mutex.wait();
			}
			locked = -1;
		}
	}

	public void releaseLock() {
		synchronized (mutex) {
			if (locked == -1)
				locked = 0;
			else
				--locked;
		}
	}

	public Document getDOM() {
		try {
		return
		  documentManager.getEnvironment().getDOMParser().parse(fileName);
		} catch(Exception e) {
			throw new FuzzyXMLUnsupportedException(e);
		}
	}
}
