// $Id: QizXQuery.java,v 1.1.1.1 2006-03-22 13:32:17 pierre Exp $

package com.senellart.pierre.fuzzyxml.query;

import java.io.StringReader;

import javax.xml.transform.Source;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;

import com.senellart.pierre.fuzzyxml.document.DocumentManager;
import com.senellart.pierre.fuzzyxml.document.FXMLDocument;
import com.senellart.pierre.fuzzyxml.document.FileFXMLDocument;
import com.senellart.pierre.fuzzyxml.exception.FuzzyXMLUnsupportedException;
import com.senellart.pierre.fuzzyxml.exception.InvalidQueryException;
import com.senellart.pierre.fuzzyxml.exception.QueryProcessingException;

import net.axyana.qizxopen.dm.XMLSerialOptions;
import net.axyana.qizxopen.util.QName;
import net.axyana.qizxopen.xquery.XQueryConnection;
import net.axyana.qizxopen.xquery.XQueryException;
import net.axyana.qizxopen.xquery.XQueryPreparedExpression;

public class QizXQuery extends Query {
	protected XQueryPreparedExpression xquery;

	QizXQuery(DocumentManager documentManager, Source source)
			throws InvalidQueryException {
		super(documentManager, source);

		QueryFactory queryFactory = documentManager.getQueryFactory();

		if (!(queryFactory instanceof QizXQueryFactory))
			throw new FuzzyXMLUnsupportedException("Invalid query factory");

		XQueryConnection connection = ((QizXQueryFactory) queryFactory)
				.getConnection();

		try {
			xquery = connection.prepareExpression(queryString);
		} catch (XQueryException e) {
			throw new InvalidQueryException(e);
		}
	}

	public Document xQueryExecute(FXMLDocument fxmlDoc)
			throws QueryProcessingException {
		try {
			if (!(fxmlDoc instanceof FileFXMLDocument))
				throw new FuzzyXMLUnsupportedException("Invalid FXMLDocument");

			String fileName = ((FileFXMLDocument) fxmlDoc).getFileName();

			// Execute XQuery query
			xquery.bindString(QName.get("document"), fileName);

			String resultSerialization=null;
			try {
				((FileFXMLDocument) fxmlDoc).getReadLock();
				resultSerialization = xquery
						.executeQuery(new XMLSerialOptions());
			} finally {
				((FileFXMLDocument) fxmlDoc).releaseLock();
			}

			return documentManager.getEnvironment().getDOMParser().parse(
					new InputSource(new StringReader(resultSerialization)));
		} catch (Exception e) {
			throw new QueryProcessingException(e);
		}
	}
}
