// $Id: QueryFactory.java,v 1.2 2007-08-24 12:39:52 pierre Exp $

package com.senellart.pierre.fuzzyxml.query;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.senellart.pierre.fuzzyxml.FuzzyXMLEnvironment;
import com.senellart.pierre.fuzzyxml.document.DocumentManager;
import com.senellart.pierre.fuzzyxml.exception.FuzzyXMLConfigurationException;
import com.senellart.pierre.fuzzyxml.exception.FuzzyXMLUnsupportedException;
import com.senellart.pierre.fuzzyxml.exception.InvalidPathExprException;
import com.senellart.pierre.fuzzyxml.exception.InvalidQueryException;
import com.senellart.pierre.fuzzyxml.pathexpr.PathExpr;
import com.senellart.pierre.fuzzyxml.pathexpr.JoinExpr;

public abstract class QueryFactory {
	protected DocumentManager documentManager;

	protected Transformer query2XQueryStylesheet;

	public QueryFactory(DocumentManager documentManager)
			throws FuzzyXMLConfigurationException {
		this.documentManager = documentManager;

		try {
			FuzzyXMLEnvironment environment = documentManager.getEnvironment();

			query2XQueryStylesheet = environment.getXSLTFactory()
					.newTransformer(
							new StreamSource(new FileInputStream(environment
									.getPath()
									+ "/xsl/query2xquery.xsl")));
		} catch (Exception e) {
			throw new FuzzyXMLConfigurationException(e);
		}
	}

	abstract public Query newQuery(DocumentManager documentManager, Source query)
			throws InvalidQueryException;

	public Query newQuery(InputStream query) throws InvalidQueryException {
		try {
			return newQuery(documentManager, new DOMSource(documentManager
					.getEnvironment().getDOMParser().parse(
							new InputSource(query))));
		} catch (SAXException e) {
			throw new InvalidQueryException(e);
		} catch (IOException e) {
			throw new FuzzyXMLUnsupportedException(e);
		}
	}

	public Query newQuery(Node n) throws InvalidQueryException {
		return newQuery(documentManager, new DOMSource(n));
	}

	public Query newQuery(String pathExpr, String joinExpr) throws InvalidPathExprException {
		Document d = pathExprToXml(pathExpr, joinExpr);
		// try { environment.serialize(d,System.err); } catch(Exception e) { }
		try {
			return newQuery(d);
		} catch (InvalidQueryException e) {
			// Should not happen
			throw new FuzzyXMLUnsupportedException(e);
		}
	}

	public Document pathExprToXml(String pathExpr, String joinExpr)
			throws InvalidPathExprException {
		DOMImplementation domImpl = documentManager.getEnvironment()
				.getDOMParser().getDOMImplementation();

		Map refMap = new HashMap();
		Document doc = PathExpr.parse(domImpl, pathExpr, refMap);

		Element joins = JoinExpr.parse(doc, joinExpr, refMap); 
		doc.getDocumentElement().appendChild(joins);

		return doc;
	}

	public Transformer getQuery2XQueryStylesheet() {
		return query2XQueryStylesheet;
	}
};
