// $Id: UpdateFactory.java,v 1.2 2007-08-24 12:39:52 pierre Exp $

package com.senellart.pierre.fuzzyxml.update;

import java.io.InputStream;
import java.io.StringReader;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.traversal.DocumentTraversal;
import org.w3c.dom.traversal.NodeFilter;
import org.w3c.dom.traversal.NodeIterator;
import org.xml.sax.InputSource;

import com.senellart.pierre.fuzzyxml.FuzzyXML;
import com.senellart.pierre.fuzzyxml.document.DocumentManager;
import com.senellart.pierre.fuzzyxml.exception.FuzzyXMLUnsupportedException;
import com.senellart.pierre.fuzzyxml.exception.InvalidPathExprException;
import com.senellart.pierre.fuzzyxml.exception.InvalidUpdateException;

/**
 * @author pierre
 * 
 */
public abstract class UpdateFactory {
	DocumentManager documentManager;

	public UpdateFactory(DocumentManager documentManager) {
		this.documentManager = documentManager;
	}

	abstract public Update newUpdate(InputStream update)
			throws InvalidUpdateException;

	abstract public Update newUpdate(Document update)
			throws InvalidUpdateException;

	public Update newUpdate(String pathExpr, String joinExpr, String[] insertions,
			double confidence) throws InvalidUpdateException {
		Document d = null;
		try {
			d = pathExprToXml(pathExpr, joinExpr, insertions, confidence);
		} catch (InvalidPathExprException e) {
			throw new InvalidUpdateException(e);
		}

		try {
			return newUpdate(d);
		} catch (InvalidUpdateException e) {
			// Should not happen
			throw new FuzzyXMLUnsupportedException(e);
		}
	}

	private class FXMLInsertFilter implements NodeFilter {

		public short acceptNode(Node n) {
			if (n.getNodeType() == Node.ELEMENT_NODE
					&& FuzzyXML.NAMESPACE_URI.equals(n.getNamespaceURI())
					&& (n.getLocalName().equals("insert")))
				return FILTER_ACCEPT;
			else
				return FILTER_SKIP;
		}
	}

	public Document pathExprToXml(String pathExpr, String joinExpr, String[] insertions,
			double confidence) throws InvalidPathExprException {
		int length = 0;

		if (insertions != null)
			length = insertions.length;

		Document insertionDocs[] = new Document[length];

		for (int i = 0; i < length; ++i) {
			try {
				if (!"".equals(insertions[i]))
					insertionDocs[i] = documentManager.getEnvironment()
							.getDOMParser().parse(
									new InputSource(new StringReader(
											insertions[i])));
			} catch (Exception e) {
				throw new InvalidPathExprException(e);
			}
		}

		Document result = documentManager.getQueryFactory().pathExprToXml(
				pathExpr, joinExpr);

		NodeIterator it = ((DocumentTraversal) result).createNodeIterator(
				result, NodeFilter.SHOW_ALL, new FXMLInsertFilter(), true);

		int i = 0;
		Node operation;
		for (operation = it.nextNode(); operation != null; operation = it
				.nextNode(), ++i) {
			if (i >= length)
				break;

			if (insertionDocs[i] != null) {
				Node root = insertionDocs[i].getDocumentElement();
				operation.appendChild(result.importNode(root,true));
			}
		}

		Element queryNode=result.getDocumentElement();
		Element updateNode=result.createElementNS(FuzzyXML.NAMESPACE_URI,"update");
		result.replaceChild(updateNode,queryNode);
		updateNode.appendChild(queryNode);
		updateNode.setAttribute(
				"confidence", String.valueOf(confidence));

		return result;
	}
}
