// $Id: XSLTUpdate.java,v 1.1.1.1 2006-03-22 13:32:17 pierre Exp $

package com.senellart.pierre.fuzzyxml.update;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;

import javax.xml.transform.Transformer;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;

import com.senellart.pierre.fuzzyxml.document.DocumentManager;
import com.senellart.pierre.fuzzyxml.document.FXMLDocument;
import com.senellart.pierre.fuzzyxml.document.FileFXMLDocument;
import com.senellart.pierre.fuzzyxml.exception.FuzzyXMLUnsupportedException;
import com.senellart.pierre.fuzzyxml.exception.InvalidUpdateException;
import com.senellart.pierre.fuzzyxml.exception.UpdateProcessingException;

public class XSLTUpdate extends Update {
	protected XSLTUpdate(DocumentManager documentManager, InputStream source)
			throws InvalidUpdateException {
		super(documentManager, source);
	}

	protected XSLTUpdate(DocumentManager documentManager, Document source)
			throws InvalidUpdateException {
		super(documentManager, source);
	}

	public Document executeAndReturn(FXMLDocument fxmlDoc)
			throws UpdateProcessingException {
		Document queryResult;

		try {
			queryResult = query.xQueryExecute(fxmlDoc);
			Document xUpdateOperation = buildXUpdate(queryResult, fxmlDoc);

			if (xUpdateOperation == null)
				return fxmlDoc.getDOM();
			else
				return applyUpdateOperationAndReturn(xUpdateOperation, fxmlDoc);
		} catch (Exception e) {
			throw new UpdateProcessingException(e);
		}
	}

	protected void applyUpdateOperation(Document updateOperation,
			FXMLDocument fxmlDoc) throws UpdateProcessingException {
		UpdateFactory updateFactory = documentManager.getUpdateFactory();

		if (!(updateFactory instanceof XSLTUpdateFactory))
			throw new FuzzyXMLUnsupportedException("Invalid Update Factory");

		if (!(fxmlDoc instanceof FileFXMLDocument))
			throw new FuzzyXMLUnsupportedException("Invalid Document Manager");

		Transformer stylesheet = ((XSLTUpdateFactory) updateFactory)
				.getXUpdateXSLTEngine();

		String fileName = ((FileFXMLDocument) fxmlDoc).getFileName();

		stylesheet.setParameter("document", fileName);

		File temp = null;
		try {
			temp = File.createTempFile("fxml", ".xml", documentManager
					.getEnvironment().getTempDir());

			((FileFXMLDocument) fxmlDoc).getWriteLock();
			stylesheet.transform(new DOMSource(updateOperation),
					new StreamResult(new FileOutputStream(temp)));
		} catch (Exception e) {
			throw new UpdateProcessingException(e);
		} finally {
			((FileFXMLDocument) fxmlDoc).releaseLock();
		}
	}

	protected Document applyUpdateOperationAndReturn(Document updateOperation,
			FXMLDocument fxmlDoc) throws UpdateProcessingException {
		UpdateFactory updateFactory = documentManager.getUpdateFactory();

		if (!(updateFactory instanceof XSLTUpdateFactory))
			throw new FuzzyXMLUnsupportedException("Invalid Update Factory");

		if (!(fxmlDoc instanceof FileFXMLDocument))
			throw new FuzzyXMLUnsupportedException("Invalid Document Manager");

		Transformer stylesheet = ((XSLTUpdateFactory) updateFactory)
				.getXUpdateXSLTEngine();

		String fileName = ((FileFXMLDocument) fxmlDoc).getFileName();

		stylesheet.setParameter("document", fileName);

		Document resultDoc = null;

		try {
			resultDoc = documentManager.getEnvironment().getDOMParser()
					.newDocument();

			((FileFXMLDocument) fxmlDoc).getReadLock();
			stylesheet.transform(new DOMSource(updateOperation), new DOMResult(
					resultDoc));
		} catch (Exception e) {
			throw new UpdateProcessingException(e);
		} finally {
			((FileFXMLDocument) fxmlDoc).releaseLock();
		}

		return resultDoc;
	}
}
